/*
 * Copyright (c) 2006-2008 ITRI. All rights reserved.
 * 
 * Module name: CITRIC cameraboard API global variables ITRI 
 *          
 * Abstact : CITRIC cameraboard API global variables ITRI
 * 
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/stat.h>
#include <linux/types.h>
#include <pthread.h>
#include <jpeglib.h>

#include "cameraboard.h"
#include "videodev.h"
#include "pxa_camera.h"
#include "citric_packet.h"

#define CAMERA_I2C_ADDR 0x30
#define CAMERA_MIDH 0x1c
#define CAMERA_MIDL 0x1d
#define CAMERA_PIDH 0x0a
#define CAMERA_PIDL 0x0b
#define CHIP_DELAY  0xFF
#define I2C_SLAVE 0x0703
//#define BAUDRATE B115200
#define BAUDRATE B38400

#define YCbCrtoR(Y,Cb,Cr) (1000*Y + 1371*(Cr-128))/1000
#define YCbCrtoG(Y,Cb,Cr) (1000*Y - 336*(Cb-128) - 698*(Cr-128))/1000
#define YCbCrtoB(Y,Cb,Cr) (1000*Y + 1732*(Cb-128))/1000
#define min(x1, x2)   (((x1)<(x2))?(x1):(x2))

#define MAX_QUE_SIZE	4096 
#define MAX_ENTRY_SIZE	128


struct reg_set_s {
  int val1;
  int val2;
};

int fd_i2c;
int fd_camera;
struct video_window vw;
struct reg_set_s reg;
__u8 *buf = NULL;
struct termios tio_camera;
fd_set rfds;
struct timeval tv;

int fd_uart; // UART port
struct termios tio_uart; // terminal settings struct

unsigned short ov_image_width; // width used by OV9655 sensor
unsigned short ov_image_height; // height used by OV9655 sensor
unsigned short ov_autoexp;
unsigned int img_res;
unsigned int img_color;

cameraboard cb[1];

unsigned long telosbtime; // telosb time at time of last sync
unsigned long cameratime; // local time at time of last sync
unsigned long globaltime;
unsigned int uptime_min;

unsigned char rcv_cbpacket[MAX_ENTRY_SIZE];
unsigned char rcv_cbpacket_length;

unsigned char cbprintf_byte;
unsigned char cbprintf_groups[8];

unsigned char g_inque[MAX_QUE_SIZE][MAX_ENTRY_SIZE];
unsigned char g_outque[MAX_QUE_SIZE][MAX_ENTRY_SIZE];
//unsigned char g_outque_id[MAX_QUE_SIZE];
//cv_struct_t* g_outque_cv_struct_ptr[MAX_QUE_SIZE];

unsigned short g_inque_start, g_inque_end, g_inque_count;
unsigned short g_outque_start, g_outque_end, g_outque_count;

pthread_mutex_t g_inque_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t g_outque_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t g_time_mutex = PTHREAD_MUTEX_INITIALIZER;

unsigned long g_global_time = 0;
unsigned long g_system_time = 0;

unsigned short g_mote_id = 0xFFFF;

#define CAMREG_VREF			0x03
#define CAMREG_COM1			0x04
#define CAMREG_COM5			0x0E
#define CAMREG_COM6			0x0F
#define CAMREG_CLKRC		0x11
#define CAMREG_COM7			0x12
#define CAMREG_COM8			0x13
#define CAMREG_COM9			0x14
#define CAMREG_REG16		0x16
#define CAMREG_HSTART		0x17
#define CAMREG_HSTOP		0x18
#define CAMREG_VSTART		0x19
#define CAMREG_VSTOP		0x1A
#define CAMREG_MVFP			0x1E
#define CAMREG_AEW			0x24
#define CAMREG_AEB			0x25
#define CAMREG_VPT			0x26
#define CAMREG_BBIAS		0x27
#define CAMREG_GBIAS		0x28
#define CAMREG_PREGAIN	0x29
#define CAMREG_RBIAS		0x2C
#define CAMREG_HREF			0x32
#define CAMREG_AREF3		0x36
#define CAMREG_ADC2			0x38
#define CAMREG_AREF4		0x39
#define CAMREG_TSLB			0x3A
#define CAMREG_COM11		0x3B
#define CAMREG_COM14		0x3E
#define CAMREG_EDGE			0x3F
#define CAMREG_COM16		0x41
#define CAMREG_COM17		0x42
#define CAMREG_MTX1			0x4F
#define CAMREG_MTX2			0x50
#define CAMREG_MTX3			0x51
#define CAMREG_MTX4			0x52
#define CAMREG_MTX5			0x53
#define CAMREG_MTX6			0x54
#define CAMREG_DBLV			0x6B	//added by leon 20080707
#define CAMREG_POIDX		0x72
#define CAMREG_PCKDV		0x73
#define CAMREG_XINDX		0x74
#define CAMREG_YINDX		0x75
#define CAMREG_COM19		0x8C
#define CAMREG_COM24		0xC7

unsigned char camera_regs[208];
unsigned char camera_regs_write[] =
{
0x11,
0x6b,
0x12,
0x36,
0x69,
0x8c,
0x3e,
0x41,
0x72,
0x73,
0x74,
0x75,
0x76,
0xc7,
0x03,
0x17,
0x18,
0x19,
0x1a,
0x32,
0xc0,
0x3a,
0x13,
0x00,
0x01,
0x02,
0x04,
0x0b,
0x0e,
0x0f,
0x10,
0xa1,
0xa6,
0x16,
0x1e,
0x29,
0xa9,
0x27,
0x28,
0x2a,
0x2b,
0x2c,
0x33,
0x34,
0x35,
0x38,
0x39,
0x3d,
0x40,
0x4a,
0x4b,
0x4c,
0x4d,
0x4e,
0x5f,
0x60,
0x61,
0x42,
0x3f,
0x76,
0x70,
0x77,
0x55,
0x56,
0x57,
0x3c,
0x7a,
0x7b,
0x7c,
0x7d,
0x7e,
0x7f,
0x80,
0x81,
0x82,
0x83,
0x84,
0x85,
0x86,
0x87,
0x88,
0x89,
0x8a,
0x90,
0x91,
0x9f,
0xa0,
0xa4,
0xa8,
0xa9,
0xaa,
0xab,
0xac,
0xad,
0xae,
0xaf,
0xb2,
0xb3,
0xb4,
0xb5,
0xb6,
0xbb,
0xbc,
0xbd,
0xbe,
0xbf,
0xc1,
0xc2,
0xc3,
0x24,
0x25,
0x26,
0xC6,
0xcd,
0x71,
0xa5,
0xca,
0xc9,
0xcB,
0xcc,
0x14,
0x3b,
0x4f,
0x50,
0x51,
0x52,
0x53,
0x54,
0x58,
0x43,
0x44,
0x45,
0x46,
0x47,
0x48,
0x59,
0x5a,
0x5b,
0x5c,
0x5d,
0x5e,
0x6c,
0x6d,
0x6e,
0x6f,
0x62,
0x63,
0x64,
0x65,
0x66,
0x9d,
0x9e,
0x3a,
0x6a,
0xc5,
0xa2,
0xa3,
0x13,
0x42,
};

void downsample_VGA(unsigned char* vgaframe, unsigned char* newframe, int factor);
void queue_init();
void cameraboard_start_commthread();
ssize_t uart_writeTOSpackets(unsigned char* payload, int length, int id, unsigned short dest);

#include "cameraboard_itri.c"
#include "cameraboard_ucb.c"
