/*
Copyright C2008. The Regents of the University of California (Regents). All
Rights Reserved. Permission to use and modify this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby granted,
provided that the above copyright notice, this paragraph and the following
two paragraphs appear in all copies, modifications, and distributions.
Contact The Office of Technology Licensing, UC Berkeley, 2150 Shattuck
Avenue, Suite 510, Berkeley, CA 94720-1620, (510) 643-7201, for commercial
licensing opportunities. IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED
HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


void downsample_VGA(unsigned char* vgaframe, unsigned char* newframe, int factor)
{
	int vga_w = 640;
	int vga_w2 = 320;
	int vga_h = 480;
	int offset = vga_w * vga_h;
	int offset2 = offset + (vga_w2 * vga_h);
	
	int new_w = vga_w / factor;
	int new_w2 = new_w / 2;
	int new_h = vga_h / factor;
	int newoffset = new_w * new_h;
	int newoffset2 = newoffset + (new_w2 * new_h);
	
	int horskip = factor;
	int vertskip = vga_w * factor;
	int vertskip2 = vga_w2 * factor;
	
	int i,j;
	
	for (i=0; i<new_h; i++) {
		for (j=0; j<new_w; j++) {
			newframe[i*new_w+j] = vgaframe[i*vertskip+j*horskip];
		}
	}
	
	for (i=0; i<new_h; i++) {
		for (j=0; j<new_w2; j++) {
			newframe[newoffset+i*new_w2+j] = vgaframe[offset+i*vertskip2+j*horskip];
		}
	}
	
	for (i=0; i<new_h; i++) {
		for (j=0; j<new_w2; j++) {
			newframe[newoffset2+i*new_w2+j] = vgaframe[offset2+i*vertskip2+j*horskip];
		}
	}
}


int enqueue_incoming(unsigned char* data, unsigned char size)
{
	if(inqueue_is_full() || size > MAX_ENTRY_SIZE)
		return 1;

	pthread_mutex_lock(&g_inque_mutex);

	*(g_inque[g_inque_end]) = size;
	memcpy(g_inque[g_inque_end] + 1, data, size);

	g_inque_end = (g_inque_end + 1) % MAX_QUE_SIZE;

	g_inque_count++;

	pthread_mutex_unlock(&g_inque_mutex);

	return 0;
}

int enqueue_outgoing(unsigned char* data, unsigned char size, unsigned char id, unsigned short dest)
{
	if(outqueue_is_full() || size > MAX_ENTRY_SIZE)
		return 1;

	pthread_mutex_lock(&g_outque_mutex);

	*(g_outque[g_outque_end]) = size;
	*(g_outque[g_outque_end]+1) = id;
	*(g_outque[g_outque_end]+2) = (dest >> 8) && 0xFF;
	*(g_outque[g_outque_end]+3) = dest & 0xFF;
	memcpy(g_outque[g_outque_end] + 4, data, size);

	// condition variables for signalling a waiting thread
	// that the packet has been sent
	//g_outque_cv_struct_ptr[g_outque_end] = cv_struct_ptr;

	g_outque_end = (g_outque_end + 1) % MAX_QUE_SIZE;

	g_outque_count++;

	pthread_mutex_unlock(&g_outque_mutex);

	return 0;
}

int dequeue_incoming(unsigned char* data, unsigned char* size)
{
	if(inqueue_is_empty())
		return 1;

	pthread_mutex_lock(&g_inque_mutex);

	*size = *(g_inque[g_inque_start]);

	memcpy(data, g_inque[g_inque_start] + 1, *size);

	g_inque_start = (g_inque_start + 1) % MAX_QUE_SIZE;

	g_inque_count--;

	pthread_mutex_unlock(&g_inque_mutex);

	return 0;
}

int dequeue_outgoing(unsigned char* data, unsigned char* size, unsigned char* id, unsigned short* dest)
{
	if(outqueue_is_empty() || size == NULL || data == NULL)
		return 1;

	pthread_mutex_lock(&g_outque_mutex);

	*size = *(g_outque[g_outque_start]);
	*id   = *(g_outque[g_outque_start]+1);
	*dest = (unsigned short)(*(g_outque[g_outque_start]+2)) << 8;
	*dest |= *(g_outque[g_outque_start]+3);
	memcpy(data, g_outque[g_outque_start] + 4, *size);

	// condition variables for signalling a waiting thread
	// that the packet has been sent
	/*
	if (g_outque_cv_struct_ptr[g_outque_start] != NULL) {
	  pthread_mutex_lock(g_outque_cv_struct_ptr[g_outque_start]->mutex_ptr);
	  pthread_cond_signal(g_outque_cv_struct_ptr[g_outque_start]->cv_ptr);
	  pthread_mutex_unlock(g_outque_cv_struct_ptr[g_outque_start]->mutex_ptr);
	}
	*/

	g_outque_start = (g_outque_start + 1) % MAX_QUE_SIZE;

	g_outque_count--;

	pthread_mutex_unlock(&g_outque_mutex);

	return 0;
}

int cbpkt_send(unsigned char type, unsigned int dest, unsigned char* payload, unsigned int length)
{
	int cbpkt_length = length + CBPKT_HEADER_LENGTH;
	unsigned char cbpkt[CBPKT_HEADER_LENGTH + CBPKT_MAX_PAYLOAD];
	
	memcpy(cbpkt + CBPKT_HEADER_LENGTH, payload, length);
	CBPKT_SET_TYPE(cbpkt,type);
	CBPKT_SET_ACTUALSRCID(cbpkt,g_mote_id);
	
	if (enqueue_outgoing(cbpkt, cbpkt_length, HANDLERID_CITRIC, dest))
		return 1;
	
	return 0;
}

int cbpkt_rcv(unsigned char* type, unsigned int* src_id_actual, unsigned char* payload, unsigned int* length)
{
	if (dequeue_incoming(rcv_cbpacket, &rcv_cbpacket_length))
	{
		return 0;
	}
	else
	{
		*type = CBPKT_GET_TYPE(rcv_cbpacket);
		*src_id_actual = CBPKT_GET_ACTUALSRCID(rcv_cbpacket);
		*length = rcv_cbpacket_length - CBPKT_HEADER_LENGTH;
		memcpy(payload, rcv_cbpacket + CBPKT_HEADER_LENGTH, *length);
		return 1;
	}
}

int inqueue_is_full()
{
//	return ((g_inque_end + 1) % MAX_QUE_SIZE == g_inque_start);
	return (g_inque_count == MAX_QUE_SIZE);
}
int outqueue_is_full()
{
//	return ((g_outque_end + 1) % MAX_QUE_SIZE == g_outque_start);
	return (g_outque_count == MAX_QUE_SIZE);
}
int inqueue_is_empty()
{
//	return (g_inque_start == g_inque_end);
	return (g_inque_count == 0);
}
int outqueue_is_empty()
{
//	return (g_outque_start == g_outque_end);
	return (g_outque_count == 0);
}

int inqueue_get_free_bytes()
{
	return (MAX_QUE_SIZE - g_inque_count);
}

int outqueue_get_free_bytes()
{
	return (MAX_QUE_SIZE - g_inque_count);
}

void queue_init()
{
	g_inque_start = g_inque_end = g_outque_start = g_outque_end = 0;
	g_inque_count = g_outque_count = 0;

	
}

void cameraboard_sync_time()
{
	unsigned char outcbpkt[MAX_ENTRY_SIZE];
	cbpkt_send(CB_TYPE_SYNC, 0, outcbpkt, 4);
}

unsigned long cameraboard_get_time()
{
	struct timeval temptime;
	unsigned long nowtime;

	gettimeofday(&temptime, NULL);
	nowtime = temptime.tv_sec * 1000;
	nowtime += temptime.tv_usec / 1000;

	if(telosbtime == 0) return nowtime;

	return (nowtime - cameratime) + telosbtime;
}

void* receive_thread(void* arg) {
	unsigned char c, c_prev;
	
	unsigned char serialpkt[MAX_ENTRY_SIZE];
	unsigned int serialpkt_length;
	unsigned int handler_id;
	unsigned short src_id;
	
	unsigned char* ampkt;
	unsigned int ampkt_length;
	
	unsigned char* cbpkt;
	unsigned int cbpkt_length;
	unsigned int cbpkt_type;
	
	unsigned char outcbpkt[MAX_ENTRY_SIZE];
	unsigned int outcbpkt_length;
	
	unsigned long in_drop = 0;
	unsigned char out_data[MAX_ENTRY_SIZE];
	unsigned char out_length = 1;
	unsigned char out_id = CB_TYPE_NOOUTGOING;
	unsigned short out_dest = 0;
	unsigned long out_drop = 0;
	unsigned long time_comp;
	int write_len;
	int i;
	struct timeval temptime;

	int errcount = 0;

	unsigned char data[MAX_ENTRY_SIZE];

	out_length = 3;
	out_id = HANDLERID_CITRIC;
	CBPKT_SET_TYPE(out_data, CB_TYPE_NOOUTGOING);
//	data[0] = CB_TYPE_NOOUTGOING;
//	CBPKT_SET_ACTUALSRCID(data, g_mote_id);

	while(1) 
	{
		errcount = 0;
		while(1) 
		{
			uart_read(&c, 1);
			if ((c == 0x7E))
				break;	
		}
		serialpkt_length = 0;

		while(1) 
		{
			uart_read(&c, 1);
			if (c == 0x7E) 
			{
				break;
			} 
			else if (c_prev == 0x7D)
			{
				if (c == 0x5D) 
				{
					serialpkt[serialpkt_length-1] = 0x7D;
				} 
				else if (c == 0x5E) {
					serialpkt[serialpkt_length-1] = 0x7E;
				}
			} 
			else 
			{
				serialpkt[serialpkt_length] = c;
				serialpkt_length++;
			}
			c_prev = c;
		}

		ampkt = serialpkt + SERIALPKT_HEADER_LENGTH;
		ampkt_length = serialpkt_length - SERIALPKT_HEADER_LENGTH - SERIALPKT_FOOTER_LENGTH;

		handler_id = AMPKT_GET_HANDLERID(ampkt);
		src_id = AMPKT_GET_SRCID(ampkt);

		if (handler_id == HANDLERID_CITRIC)
		{
			cbpkt = ampkt + AMPKT_HEADER_LENGTH;
			cbpkt_length = ampkt_length - AMPKT_HEADER_LENGTH;
			cbpkt_type = CBPKT_GET_TYPE(cbpkt);

			switch(cbpkt_type)
			{
				case CB_TYPE_PINGREQ:
					globaltime = cameraboard_get_time();
					struct timeb uptime;
					ftime(&uptime);
					uptime_min = uptime.time / 60;
					outcbpkt[0] = 0x00;
					outcbpkt[1] = 0x00;
					outcbpkt[2] = 0x00;
					outcbpkt[3] = 0x00;
					outcbpkt[4] = (globaltime >> 24) & 0xFF;
					outcbpkt[5] = (globaltime >> 16) & 0xFF;
					outcbpkt[6] = (globaltime >> 8) & 0xFF;
					outcbpkt[7] = globaltime & 0xFF;
					outcbpkt[8] = 0x00;
					outcbpkt[9] = g_inque_count;
					outcbpkt[10] = g_outque_count;
					outcbpkt[11] = (uptime_min >> 24) & 0xFF;
					outcbpkt[12] = (uptime_min >> 16) & 0xFF;
					outcbpkt[13] = (uptime_min >> 8) & 0xFF;
					outcbpkt[14] = uptime_min & 0xFF;
					outcbpkt[15] = img_res & 0xFF;
					outcbpkt[16] = img_color & 0xFF;
					outcbpkt[17] = ov_autoexp & 0xFF;
					outcbpkt[18] = 0x00;
					outcbpkt[19] = 0x00;
					
					cbpkt_send(CB_TYPE_PING, 0, outcbpkt, 20);
					
					break;
				case CB_TYPE_PING:
					break;
				case CB_TYPE_SET:
					break;
				case CB_TYPE_PRINTFCONFIG:
					cbprintf_byte = cbpkt[CBPKT_HEADER_LENGTH];
					for (i=0; i<8; i++)
					{
						cbprintf_groups[i] = cbprintf_byte & 0x01;
						cbprintf_byte = cbprintf_byte >> 1;
					}
					break;
				case CB_TYPE_PRINTF:
					break;
				case CB_TYPE_FILEINFO:
					break;
				case CB_TYPE_FILE:
					break;
				case CB_TYPE_STOPTIMESYNC:
					break;
				case CB_TYPE_SYNC:
					gettimeofday(&temptime, NULL);
					cameratime = temptime.tv_sec * 1000;
					cameratime += temptime.tv_usec / 1000;
					telosbtime = (cbpkt[CBPKT_HEADER_LENGTH] << 24) + (cbpkt[CBPKT_HEADER_LENGTH+1] << 16) + (cbpkt[CBPKT_HEADER_LENGTH+2] << 8) + cbpkt[CBPKT_HEADER_LENGTH+3];
//printf("telosb time = %d, local time = %d\n", telosbtime, cameratime);
					break;
				case CB_TYPE_RETRANSREQ:
					break;
				case CB_TYPE_TIMECOMP:
					break;
				case CB_TYPE_NOINCOMING:
					if(g_mote_id == 0xFFFF)
					{
						g_mote_id = src_id;
						printf("Mote ID : %04X\n", g_mote_id);
					}
					break;
				case CB_TYPE_TIMEREFLECT:
					break;
				default:
					//cut out MSG_TYPE and SEQ of serial communication.
					if( enqueue_incoming(cbpkt, cbpkt_length) )
					{
						printf("Packet drop from recv thread!!\n");
						in_drop++;
					}
					break;
			}
			
			if(cbpkt_type == CB_TYPE_RETRANSREQ)
			{
				//for data retransmisson
				printf("retrans id(%d), type(%02X) len(%d)\n", out_id, out_data[0], out_length);
			}
			else
			{
				if( dequeue_outgoing(out_data, &out_length, &out_id, &out_dest) == 0)
				{
					//have normal data to send
				}
				else
				{
					//have nothing to send
					out_length = 3;
					out_id = HANDLERID_CITRIC;
					CBPKT_SET_TYPE(out_data, CB_TYPE_NOOUTGOING);
					CBPKT_SET_ACTUALSRCID(out_data, g_mote_id);
				}
			}

			write_len = uart_writeTOSpackets(out_data, out_length, out_id, out_dest);
		}
	}
}

void cameraboard_start_commthread()
{
printf("cameraboard start commthread\n");
	pthread_t thread_rcv;
	int pthread_errorcode;
	if (pthread_errorcode = pthread_create(&thread_rcv, NULL, receive_thread, 0))
	{
		printf("error creating receive thread %d\n", pthread_errorcode);
	}
}


void camera_set_autoexp(int x) {
	ov_autoexp = x;
	if (x) {
		if(i2c_write(fd_i2c, CAMREG_COM8, 0x8F)) printf("set auto exp fail!");
	} else {
		if(i2c_write(fd_i2c, CAMREG_COM8, 0x88)) printf("set auto exp fail!");
	}
}


int camera_board_is_sync()
{
	return (g_mote_id != 0xFFFF);
}


void camera_calibrate_exp(int calibration_time, int auto_exp_on, int verbose) {
	int i,j;
	struct timeb time_a;
	
	if (verbose) printf("calibrating exposure\n");
	
	camera_set_autoexp(1); // turn on auto exposure
	
	ftime(&time_a);
	j = time_a.time;
	for(i=0; i<calibration_time; i++) {
		if (verbose) printf("%d\n",(calibration_time-i));
		while(time_a.time == j) ftime(&time_a); // wait till next second
		j = time_a.time;
		camera_capture_frame();
	}
	
	if (!auto_exp_on) camera_set_autoexp(0); // turn off auto exposure
	
	if (verbose) printf("done\n");
}

ssize_t uart_read(unsigned char* packet, int length) {
	return read(fd_uart, packet, length);
}

ssize_t uart_write(unsigned char* packet, int length) {
	return write(fd_uart, packet, length);
}

unsigned short crcByte(unsigned short crc, unsigned char b) {
  crc = (unsigned char)(crc >> 8) | (crc << 8);
  crc ^= b;
  crc ^= (unsigned char)(crc & 0xff) >> 4;
  crc ^= crc << 12;
  crc ^= (crc & 0xff) << 5;
  return crc;
}

ssize_t uart_writeTOSpackets(unsigned char* payload, int length, int id, unsigned short dest) {
	int i,j;
	unsigned char tospacket[256];
	unsigned char serialpacket[256];
	int tospacketindex;
	int serialindex;
	int payloadindex;
	ssize_t ret;
	unsigned short crc = 0;

	static unsigned char seq;

	tospacket[0] = 0x7e;
	tospacket[1] = 0x45;
	tospacket[2] = seq;
	tospacket[3] = 0x00;
	tospacket[4] = (dest >> 8) & 0xFF;
	tospacket[5] = dest & 0xFF;
	tospacket[6] = (g_mote_id >> 8) & 0xFF;
	tospacket[7] = g_mote_id & 0xFF;
	tospacket[8] = (unsigned char)length;
	tospacket[9] = 0x00;
	tospacket[10] = (unsigned char)id;

	for(i=1;i<11;i++)
		crc = crcByte(crc, tospacket[i]);

	tospacketindex = 11;

	for(i=0;i<length;i++)
	{
      		tospacket[tospacketindex++] = payload[i];
		crc = crcByte(crc, payload[i]);	
	}	

	//printf("len %d , payload %d bytes send with CRC %02X %02X \n", tospacketindex, length, tospacket[tospacketindex-2], tospacket[tospacketindex-1]);

	// tos packet footer
	tospacket[tospacketindex++] = (crc >> 8) & 0xFF;
	tospacket[tospacketindex++] = crc & 0xFF;
	tospacket[tospacketindex++] = 0x7e;

	// build serial frame with delimiters and special characters
	serialindex=0;

	serialpacket[serialindex++] = 0x7E;

	for(i=1;i<tospacketindex-1;i++)
	{
		if(tospacket[i] == 0x7E)
		{
			serialpacket[serialindex++] = 0x7D;
			serialpacket[serialindex++] = 0x5E;
		}
		else if(tospacket[i] == 0x7D)
		{
			serialpacket[serialindex++] = 0x7D;
			serialpacket[serialindex++] = 0x5D;
		}
		else
			serialpacket[serialindex++] = tospacket[i];
	}	
	
	serialpacket[serialindex++] = 0x7E;
/*
	for(i=0;i<serialindex;i++)
		printf("%02X ", serialpacket[i]);
	printf("\n");
*/

	ret = uart_write(serialpacket, serialindex);
/*
	if(id != PACKET_TYPE_NOOUTGOING)
		printf("Payload size (%d), TOS packet size (%d), Actual frmae size(%d) has been sent.\n", length, tospacketindex, serialindex);
*/
	seq++;
	return ret;
}

void write_binary_image(unsigned char* buf, int image_width, int image_height, char* fn, int color) {
	int i, j;
	int image_length = image_width * image_height;
	
	printf("writing %s\n", fn);
	
	FILE* fp = fopen(fn,"w");
	
	for (i=0; i<image_length; i++) {
			fputc(buf[i], fp);
	}
	
	if (color) {
		for (i=image_length; i<(image_length*2); i++) {
			fputc(buf[i], fp);
		}
	}

	fclose(fp);
}


int get_cbprintf_group(int groupnum)
{
	return (int) cbprintf_groups[groupnum];
}

void set_cbprintf_group(int groupnum, int value)
{
	cbprintf_groups[groupnum] = value;
}

void cbprintf(int groupnum, char* format, ...)
{
	int i;
	va_list args;
	char s[MAX_ENTRY_SIZE];
	
	if (cbprintf_groups[groupnum])
	{
		va_start(args, format);
		vsprintf(s, format, args);
		va_end(args);
		
		for (i=0; i<CBPKT_MAX_PAYLOAD; i++)
		{
			if (s[i] == 0) break;
		}
		
		cbpkt_send(CB_TYPE_PRINTF, 0, s, i);
	}
}

void send_file(int dest, char* fn)
{
	int i;
	unsigned char datapkt[CBPKT_MAX_PAYLOAD];
	int chunksize = CBPKT_MAX_PAYLOAD - 2;
	
	struct stat file_status;
	if (stat(fn, &file_status))
	{
		printf("error getting file size\n");
		return 1;
	}
	
	int fn_length = strlen(fn);
	unsigned int filesize = (unsigned int) file_status.st_size;
	unsigned char infopkt[MAX_ENTRY_SIZE];
	FILE* f;
	
	f = fopen(fn, "rb");
	if (f == NULL)
	{
		printf("error opening file\n");
		return 1;
	}
	
	infopkt[0] = 0;
	infopkt[1] = (filesize >> 24) & 0xFF;
	infopkt[2] = (filesize >> 16) & 0xFF;
	infopkt[3] = (filesize >> 8) & 0xFF;
	infopkt[4] = filesize & 0xFF;
	memcpy(infopkt+5, fn, fn_length);
	cbpkt_send(CB_TYPE_FILEINFO, dest, infopkt, fn_length+5); // send start pkt
	
//	printf("chunksize = %d, filesize = %d\n", chunksize, filesize);
	
	i = 0;
	while (filesize >= chunksize)
	{
		datapkt[0] = (i >> 8) & 0xFF;
		datapkt[1] = i & 0xFF;
		fread(datapkt+2, 1, chunksize, f);
		
		cbpkt_send(CB_TYPE_FILE, dest, datapkt, CBPKT_MAX_PAYLOAD);
		
		filesize -= chunksize;
		i++;
	}
	if (filesize != 0)
	{
		datapkt[0] = (i >> 8) & 0xFF;
		datapkt[1] = i & 0xFF;
		fread(datapkt+2, 1, filesize, f);
		
		cbpkt_send(CB_TYPE_FILE, dest, datapkt, 2+filesize);
	}
	
	fclose(f);
	
	infopkt[0] = 1;
	cbpkt_send(CB_TYPE_FILEINFO, dest, infopkt, fn_length+5); // send end pkt
}
