// demo app for testing camera settings

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>
#include "cameraboard.h"

// maximum number of blobs per frame
#define MAX_BLOBS		128

#define BASESTATION_ID 0

#define diff(x,y) ((x>y)?(x-y):(y-x))


void print_help() {
	printf("%4s %-10s %s\n", "-r", "<value>", "resolution: (default 1)");
	printf("%4s %-10s %s\n", "", "", "0 - VGA 640x480");
	printf("%4s %-10s %s\n", "", "", "1 - QVGA 320x240");
	printf("%4s %-10s %s\n", "", "", "2 - QQVGA 160x120");
	printf("%4s %-10s %s\n", "-a", "", "turn on auto-exposure (default off)");
	printf("%4s %-10s %s\n", "-w", "<value>", "exposure warmup time (default 5)");
	printf("%4s %-10s %s\n", "-g", "", "grayscale images (default color images)");
}


int main(int argc, char *argv[]) {
	int i,j;
	
	struct timeb time_a, time_b;
	int time_diff;
	
	char* filename[16];

	int res = 1;
	int autoexp = 0;
	int warmup_time = 5;
	int color_img = 1;
	
	// process command line args
	i = 1;
	while (i < argc) {
		char* arg = argv[i];
		
		if (arg[0] == '-') {
			if (arg[1]) {
				switch(arg[1]) {
					case 'r':
						if (argc > (i+1)) {
							j = atoi(argv[i+1]);
							res = j;
							i++;
						}
						break;
					case 'a':
						autoexp = 1;
						break;
					case 'w':
						if (argc > (i+1)) {
							j = atoi(argv[i+1]);
							warmup_time = j;
							i++;
						}
						break;
					case 'g':
						color_img = 0;
						break;
					case '-':
						print_help();
						return 0;
						break;
				}
			}
		}
		
		i++;
	}
	
	cameraboard* cb = cameraboard_open(IMG_COLOR_YUV_422, res, autoexp);
	
	camera_calibrate_exp(warmup_time, autoexp, 1);

	cameraboard_sync_time();
	
	i = 0;
	ftime(&time_a); // init timing
	while(1) {
		printf("press any key to capture image, ESC to exit\n");
		if (1) { // wait for key press
			char cmd;
			read(0, &cmd, 1); // read char from std input
			if (cmd == 0x1B) break; // ESC char, exit loop
		}
		
		if (camera_capture_frame()) {
			printf("camera_get_frame error\n");
		}
		
		memset(filename, 0, sizeof(filename));
		sprintf(filename,"im%04d.jpg", i);
		
		write_jpeg_image(cb->img, cb->img_width, cb->img_height, filename, color_img, 80);
		send_file(0, filename);

		// calculate time per frame
		ftime(&time_b); // stop timing
		time_diff = (time_b.time - time_a.time)*1000 + (time_b.millitm - time_a.millitm);
		time_a.time = time_b.time; // reset time_a to beginning of next iteration
		time_a.millitm = time_b.millitm; // reset time_a to beginning of next iteration
		
		// print img capture time
		printf("img capture time = %d\n", cb->img_time);
		// print iteration time in ms
		//printf("frame %4d - %d\n", i, time_diff);
		
		i++;
	}
	
	cameraboard_close();
}
