#include <iostream>
#include <fstream>
#include <string>
#include "_MoteComm.h"

#ifdef __linux
	#include <string.h>
	#include "_LinuxSocket.h"
#else
	#include "_Socket.h"
#endif

#include "_ServerMenu.h"
#include "_TaskManager.h"
#include "_UserTask.h"
using namespace std;


int main(int argCt, char* argv[])
{
	ifstream in;
	char buf[1000];
	char data[1000];
	int dataSz;
	int tmp;
	char tmp2[128];
	
	// Getting Port
	for(int i = 1; i<(argCt-1); i++)
		if(argv[i][0]=='-' && argv[i][1]=='p')
		{
			strcpy(MCOMM_PORT,argv[i+1]);
		}
	printf("Using Port: %s\n\n",MCOMM_PORT);

	// Starting Process for connecting with mote.
	StartMoteConnection();
	// Starting processing task for packets.
	StartTaskManager();

	// Start TCP socket for clients
	startSocket();

	// Entering main Menu
	int ch_main = -1;
	while(ch_main!=MENU_MAIN_EXIT)
	{
		// Getting input from main menu.
		ch_main = InputMainMenu();

		switch(ch_main)
		{
		// [ SELECT MOTES OPTIONS ]
		// This option is used to bring a menu with options for the selected motes.
		// For example we can specify what motes are selected or save the current
		// set of selected motes. The set of selected motes is used for display and
		// setting operations.
		case MENU_MAIN_SELECTION:
			InputMoteSelectionMenu();
			break;

		// [ DISPLAY OPTIONS ]
	  // Here we take care of the display options for the stream of data packet
		// from the motes. The user can choose how the packets will be displayed.
		// Display will continue until the <ENTER> key is pressed.
		case MENU_MAIN_DISPLAY:
			switch(InputDisplayMenu())
			{
				case MENU_DISPLAY_DEC: // Displaying packets as decimals
					MCOMM_DISPLAY = MCOMM_DISPLAY_DEC;
					cin.getline(buf,1000);
					cin.getline(buf,1000);
					MCOMM_DISPLAY = MCOMM_DISPLAY_OFF;
					break;
				case MENU_DISPLAY_HEX: // Displaying packets as hexadecimals
					MCOMM_DISPLAY = MCOMM_DISPLAY_HEX;
					cin.getline(buf,1000);
					cin.getline(buf,1000);
					MCOMM_DISPLAY = MCOMM_DISPLAY_OFF;
					break;
				case MENU_DISPLAY_CHAR: // Displaying packets as characters
					MCOMM_DISPLAY = MCOMM_DISPLAY_CHAR;
					cin.getline(buf,1000);
					cin.getline(buf,1000);
					MCOMM_DISPLAY = MCOMM_DISPLAY_OFF;
					break;
				case MENU_DISPLAY_PRINTF: // Displaying debugging packets
					TASKM_DEBUG_DISPLAY = 1;
					cin.getline(buf,1000);
					cin.getline(buf,1000);
					TASKM_DEBUG_DISPLAY = 0;
			}
			break;

    // [ LOG OPTIONS ]
    // This menu option allows the user to start the log process for any
    // packet received from motes.
    case MENU_MAIN_LOG:
      switch(InputLogMenu())
      {
      case MENU_LOG_START: // Starting the log process
        MCOMM_LOG = 1;
        break;
      case MENU_LOG_STOP: // Stopping the log process
        MCOMM_LOG = 0;
        break;
      }
      break;

	  // [ SEND PACKET ]
		// Options for sending different type of packets to the motes. It
		// includes sending stop synchronization signal and pinging a mote.
		case MENU_MAIN_SEND:
			switch(InputSendMenu())
			{
			case MENU_SEND_STOPSYNCH: // Sending stop synchronization signal
				data[ 0] = 0x7E;
				data[ 1] = 0x45;
				data[ 2] = 0x00;
				data[ 3] = 0x00;
				data[ 4] = 0xFF;
				data[ 5] = 0xFF;
				data[ 6] = 0x00;
				data[ 7] = 0x00;
				data[ 8] = 0x03;
				data[ 9] = 0x00;
				data[10] = 0x77;
				data[11] = 0x07;
				data[12] = 0x00;
				data[13] = 0x00;
				data[14] = 0x0F;
				data[15] = 0xA3;
				data[16] = 0x7E;

				SendToMote(data,17);
				break;

			case MENU_SEND_PING: // Sending ping message to selected motes
				for(int i = 0; i<SMENU_MOTELISTSZ; i++)
				{
					dataSz = CreateTinyOSMsg(data,SMENU_MOTELIST[i],PACKET_PING_OUT,0,0);
					SendToMote(data,dataSz);
				}
				break;

			case MENU_SEND_SETPRINTF: // Sending a set conf. for wireless printf
				cin.getline(buf,1000);
				printf("    Enter binary mask (XXXX XXXX) to be applied: ");
				cin.getline(buf,1000);

				tmp2[0] = 0;
				if(buf[0]=='1' || buf[0]=='0'){ tmp2[0]=tmp2[0]*2+buf[0]-48; }else{ break; }
				if(buf[1]=='1' || buf[1]=='0'){ tmp2[0]=tmp2[0]*2+buf[1]-48; }else{ break; }
				if(buf[2]=='1' || buf[2]=='0'){ tmp2[0]=tmp2[0]*2+buf[2]-48; }else{ break; }
				if(buf[3]=='1' || buf[3]=='0'){ tmp2[0]=tmp2[0]*2+buf[3]-48; }else{ break; }
				if(buf[4]!=' '){ break; }
				if(buf[5]=='1' || buf[5]=='0'){ tmp2[0]=tmp2[0]*2+buf[5]-48; }else{ break; }
				if(buf[6]=='1' || buf[6]=='0'){ tmp2[0]=tmp2[0]*2+buf[6]-48; }else{ break; }
				if(buf[7]=='1' || buf[7]=='0'){ tmp2[0]=tmp2[0]*2+buf[7]-48; }else{ break; }
				if(buf[8]=='1' || buf[8]=='0'){ tmp2[0]=tmp2[0]*2+buf[8]-48; }else{ break; }

				for(int i = 0; i<SMENU_MOTELISTSZ; i++)
				{
					dataSz = CreateTinyOSMsg(data,SMENU_MOTELIST[i],PACKET_PRINTF_OUT,tmp2,1);
					SendToMote(data,dataSz);
				}

				printf("[-]  Set Wireless PRINTF transmitted\n");
				break;

			// Sending from a file. The expected format is to have as first entry
			// the number of bytes of the message, then the content of the message.
			// Several packets can be included in the same file.
			case MENU_SEND_FILE:
				cin.getline(buf,1000);
				printf("    Name of file with messages: ");
				cin.getline(buf,1000);

				in.open(buf);
				if(in.fail())
					printf("[-] FAILED\n");
				else
				{
					// We transmit as many messages are there are in the file.
					while(1)
					{
						in>>dataSz;
						if(in.eof())
							break;

						for(int i = 0; i<dataSz; i++)
						{
							in>>tmp;
							data[i] = tmp;
						}
						SendToMote(data,dataSz);
					}
				}
				break;
			}
			break;

		// [ DISPLAY STATUS ]
		// This option display the status of all of the active motes in the system.
		// The status includes last packet received and mode of operation for each
		// mote.
		case MENU_MAIN_STATUS:
			TASKM_STATES.display();
			break;

		// [ RUN MOTION DETECTION SAMPLE PROCESS ]
		// Here we run a sample process where motion detection results from a mote
		// are forwarded to the server. This code displays results until another
		// key is pressed.
		case MENU_MAIN_RUNSAMPLE:
			USERTASK_DISPLAY = 1;
			cin.getline(buf,1000);
			cin.getline(buf,1000);
			USERTASK_DISPLAY = 0;
			break;
		}
	}

	return 0;
}
