#include "_MoteComm.h"

#ifdef __linux

	#include <pthread.h>
	#include <memory.h>

	// Header files for serial access
	#include <sys/types.h>
	#include <sys/stat.h>
	#include <fcntl.h>
	#include <termios.h>
	#include <stdio.h>
	#include <stdlib.h>

	#include "_LinuxSocket.h"

	#define HANDLE int
	#define TCHAR char
	#define TEXT(x) (x)
	#define BOOL short
	#define DWORD long
	#define WORD int

	unsigned short
	crc_packet (unsigned char *data, int len);

#endif

int MCOMM_DISPLAY = 0;
int MCOMM_LOG = 0;

#ifdef __linux
	char MCOMM_PORT[100] = "/dev/ttyUSB0";
#else
	char MCOMM_PORT[100] = "COM9";
#endif


HANDLE MCOMM_HCOM = 0; // Handle for COM port

#ifdef __linux
	struct termios oldtio, newtio;
#endif

ofstream MCOMM_OUT; // Handle to file writer
unsigned int MCOMM_OUT_COUNTER = 1; // Counter used when writing to log file

/*! Array storing the messages to be transmitted. */
xshort MCOMM_MSGBUF[QUEUE_SIZE][MAX_PACKET_LEN];

/*! Array storing the size of the messages to be transmitted. */
int MCOMM_MSGBUFSZ[QUEUE_SIZE];

/*! Variable storing the position to add the next packet to the buffer. */
int MCOMM_MSGBUFPOS = 0;

#ifdef __linux
	void* _StartMoteConnection(void* parm)
#else
	void _StartMoteConnection(void* parm)
#endif
{
  TCHAR *m_sComPort = TEXT(MCOMM_PORT);

  //--------------------------------------------------------
	// SETTING THE PORT
  //--------------------------------------------------------
  BOOL     m_bPortReady;
#ifdef __linux
#else
	DCB      m_dcb;
#endif
	BOOL     bReadRC;
#ifdef __linux
	WORD iBytesRead;
#else
	DWORD iBytesRead;
#endif

#ifdef __linux
	MCOMM_HCOM = open(m_sComPort, O_RDWR | O_NOCTTY);
  if (MCOMM_HCOM < 0) {
    fprintf(stderr, "Cannot open %s for serial i/o\n\n", m_sComPort);
    exit(1);
  }

  bzero(&newtio, sizeof(newtio));
  //newtio.c_cflag = B115200 | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_cflag = CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR | IGNBRK;
  cfsetispeed(&newtio, B115200);
  cfsetospeed(&newtio, B115200);
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;

  // Timings : http://www.unixwiz.net/techtips/termios-vmin-vtime.html
  newtio.c_cc[VTIME] = 0; // # 1/10 s between characters
  // Set VMIN to 1 so that its a blocking call unless 1 byte is in the line
  newtio.c_cc[VMIN] = 1; // 0-255 : returns that much bytes when its given

  tcgetattr(MCOMM_HCOM, &oldtio);
  tcflush(MCOMM_HCOM, TCIFLUSH);
  tcsetattr(MCOMM_HCOM, TCSANOW, &newtio);
#else
	MCOMM_HCOM = CreateFile(m_sComPort, 
			GENERIC_READ | GENERIC_WRITE,
			0,//FILE_SHARE_WRITE, // exclusive access
			0,//NULL, // no security
			OPEN_EXISTING,
			NULL,//0, // no overlapped I/O
			0);//NULL); // null template 

	m_bPortReady = SetupComm(MCOMM_HCOM, 128, 128); // set buffer sizes

	m_bPortReady = GetCommState(MCOMM_HCOM, &m_dcb);
	m_dcb.BaudRate =115200;
	m_dcb.ByteSize = 8;
	m_dcb.Parity = NOPARITY;
	m_dcb.StopBits = ONESTOPBIT;
	m_dcb.fAbortOnError = TRUE;

	m_bPortReady = SetCommState(MCOMM_HCOM, &m_dcb);

	// Retrieve the timeout parameters for all read and write operations
	// on the port. 
	COMMTIMEOUTS CommTimeouts;
	GetCommTimeouts(MCOMM_HCOM, &CommTimeouts);

	// Change the COMMTIMEOUTS structure settings.
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	CommTimeouts.ReadTotalTimeoutConstant = 0;

	// Setting the timeout parameters
	SetCommTimeouts (MCOMM_HCOM, &CommTimeouts);
#endif
	
  //--------------------------------------------------------
  // OPENNING LOG FILE
  //--------------------------------------------------------
	MCOMM_OUT.open("serverLog.txt");

  //--------------------------------------------------------
  // READING DATA
  //--------------------------------------------------------
  unsigned char packet_data[MAX_PACKET_LEN]; // Data in the packet.
  int packet_length; // Size of the packet.
  unsigned char c = 0;
  unsigned char c_prev = 0;

	unsigned char pkt[MAX_PACKET_LEN];
	int pktSz;
	int pktType;
	int moteID;

#ifdef __linux
	unsigned char serialBuffer[256];

  int inPacket = 0;
  int isEscape = 0;
  while (1) {
    // Try to get 256 characters at a time
    iBytesRead = read(MCOMM_HCOM, serialBuffer, 256);
		// echo to TCP Socket
		writeSocket(serialBuffer, 256);
    //printf("* L:%d\n", iBytesRead);

    // Digest the received characters
    int serialPtr = 0;
    while (serialPtr<iBytesRead) {
      c = serialBuffer[serialPtr++];
      if (c == 0x7E)
			{ // Delimiter 7E
				// There could be an issue with syncing if for some reason, 
				// a starting 7E converted into an ending 7E or vice versa
				if (inPacket == 0)
				{
				  inPacket = 1;
				  packet_length = 0;
				  isEscape = 0;
				}
				else
				{
				  // Done?
				  if (packet_length>0) // this is to prevent desync
				    // Since if this happens, it means we have 7E 7E 
				    // i.e. twice in a row 
				    // In that case, assume that the second 7E is the real start
				    inPacket = 2;
				}
      }
			else if (inPacket == 1)
			{
				// Are we in escape mode?
				if (isEscape == 1)
				{
				  if (c == 0x5D) // really 0x7D
				    packet_data[packet_length++] = 0x7D;
				  else if (c == 0x5E) // really 0x7E
				    packet_data[packet_length++] = 0x7E;
				  // else
				  // for now, it seems that the previous character is discarded?
				  isEscape = 0;
				}
				else
				{
				  if (c == 0x7D) // ESC 7D
				    isEscape = 1;
				  else
				    packet_data[packet_length++] = c;
				}
      }
      if ( (inPacket == 2) || (packet_length>=MAX_PACKET_LEN) )
			{
				// done with packet
				inPacket = 0;

				// Now we display the packets as they arrive.
				switch (MCOMM_DISPLAY)
				{
				case MCOMM_DISPLAY_HEX:
					printf ("<==");
					for (int i = 0; i < packet_length; i++)
						printf (" %02X", (int) packet_data[i]);
					printf ("\n");
					break;
				case MCOMM_DISPLAY_DEC:
					printf ("<==");
					for (int i = 0; i < packet_length; i++)
						printf (" %03d", (int) packet_data[i]);
					printf ("\n");
					break;
				case MCOMM_DISPLAY_CHAR:
					printf ("<==");
					for (int i = 0; i < packet_length; i++)
						printf ("%c", (char) packet_data[i]);
					printf ("\n");
					break;
				}

				// We output data into the log file.
				if (MCOMM_LOG)
				{
					MCOMM_OUT << MCOMM_OUT_COUNTER;
					for (int i = 0; i < packet_length; i++)
						MCOMM_OUT << " " << (int) packet_data[i];
					MCOMM_OUT << "\n";
				}
			    
				// check CRC?
				if (packet_length>=2)
				{
				  unsigned short pktcrc = packet_data[packet_length-1];
				  pktcrc = (pktcrc << 8) | packet_data[packet_length-2];
				  unsigned short chkcrc = crc_packet(packet_data, packet_length-2);
				  if (pktcrc != chkcrc)
					{
				    fprintf(stderr, "CRC check failed [pkt=%04x chk=%04x]\n",
					    pktcrc, chkcrc);
				  }
					else
					{
				    // Processing packet if they are packets from our protocol
				    if (packet_data[0] == 0x45)
						{
							ProcessPacket (packet_data, packet_length);
						}
				  }
				}
				else
				{
				  fprintf(stderr, "Bad packet %d\n", packet_length);
				}

				MCOMM_OUT_COUNTER++;
      }
    }
  }

  tcsetattr(MCOMM_HCOM, TCSANOW, &oldtio);
  close(MCOMM_HCOM);
  MCOMM_OUT.close ();
  pthread_exit (NULL);
#else
	
	while (1) {
	  // Find packet start delimiter 7E 45 = 126 69
	  while(1) {
      bReadRC = ReadFile(MCOMM_HCOM, &c, 1, &iBytesRead,NULL);
			if(iBytesRead)
			{
				if (c == 126){ break; }
				c_prev = c;
			}
	  }

    // Extracting rest of packet
	  packet_length = 0;
	  while(1)
    {
      bReadRC = ReadFile(MCOMM_HCOM, &c, 1, &iBytesRead,NULL);
			if(iBytesRead)
			{
				if (c == 126){ break; }
				else if (c_prev == 125)
				{ // Special char 7D
					if (c == 93)
					{ // Saw sequence 7D 5D, convert to 7D
						packet_data[packet_length-1] = 125;
					}
					else if (c == 94)
					{ // Saw sequence 7D 5E, convert to 7E
						packet_data[packet_length-1] = 126;
					}
				}
				else
				{
					packet_data[packet_length] = c;
					packet_length++;
				}
				c_prev = c;

				if (packet_length >= MAX_PACKET_LEN){ break; }
			}
	  }

		// Now we display the packets as they arrive.
		switch(MCOMM_DISPLAY)
		{
		case MCOMM_DISPLAY_HEX:
			printf("...");
			for(int i = 0; i<packet_length; i++)
				printf(" %02X",(int)packet_data[i]);
			printf("\n");
			break;
		case MCOMM_DISPLAY_DEC:
			printf("...");
			for(int i = 0; i<packet_length; i++)
				printf(" %03d",(int)packet_data[i]);
			printf("\n");
			break;
		case MCOMM_DISPLAY_CHAR:
			printf("...");
			for(int i = 0; i<packet_length; i++)
				printf("%c",(char)packet_data[i]);
			printf("\n");
			break;
		}

    // We output data into the log file.
    if(MCOMM_LOG)
    {
      MCOMM_OUT<<MCOMM_OUT_COUNTER;
      for(int i = 0; i<packet_length; i++)
        MCOMM_OUT<<" "<<(int)packet_data[i];
      MCOMM_OUT<<"\n";
    }

	  // Processing packet if they are packets from our protocol
		if(packet_data[0]==0x45)
			ProcessPacket(packet_data,packet_length);

    MCOMM_OUT_COUNTER++;
  }

	CloseHandle(MCOMM_HCOM);
  MCOMM_OUT.close();
#endif
}

#ifdef __linux
	void* _StartMoteTransmit(void* parm)
#else
	void _StartMoteTransmit(void* parm)
#endif
{
	int msgToTransmit = 0; // Keeps track of the message to be transmitted.
	xshort* msg; // Pointer to the message
	int msgSz; // Size of the message

	DWORD iBytesWritten;
	BOOL  bWriteRC;

	while(1)
	{
		// Waits while the next packet to be processed has not been added yet.
		while(msgToTransmit==MCOMM_MSGBUFPOS)
		{
#ifdef __linux
			sleep(1);
#else
			Sleep(1000);
#endif
		}

		// Getting message
		msg = MCOMM_MSGBUF[msgToTransmit];
		msgSz = MCOMM_MSGBUFSZ[msgToTransmit];

		if(MCOMM_HCOM!=0)
		{
#ifdef __linux
			iBytesWritten = write(MCOMM_HCOM, msg, msgSz);
#else
			bWriteRC = WriteFile(MCOMM_HCOM, msg,msgSz,&iBytesWritten,NULL);
#endif
		}
		else
		{
#ifdef __linux
			iBytesWritten = 0;
#else
#endif
		}

		// Now we display the transmitted message.
		switch(MCOMM_DISPLAY)
		{
		case MCOMM_DISPLAY_HEX:
			printf("...");
			for(int i = 0; i<msgSz; i++)
				printf(" %02X",(xshort)msg[i]);
			printf("\n");
			break;
		case MCOMM_DISPLAY_DEC:
			printf("...");
			for(int i = 0; i<msgSz; i++)
				printf(" %03d",(xshort)msg[i]);
			printf("\n");
			break;
		case MCOMM_DISPLAY_CHAR:
			printf("...");
			for(int i = 0; i<msgSz; i++)
				printf("%c",msg[i]);
			printf("\n");
			break;
		}

		// We output data into the log file.
		xshort tmp;
		if(MCOMM_LOG)
		{
			MCOMM_OUT<<MCOMM_OUT_COUNTER;
			for(int i = 0; i<msgSz; i++)
			{
				tmp = (xshort)msg[i];
				MCOMM_OUT<<" "<<(int)tmp;
			}
			MCOMM_OUT<<"\n";
		}
		MCOMM_OUT_COUNTER++;

		// Pointing to the next packet.
		msgToTransmit = (msgToTransmit+1)%QUEUE_SIZE;

	}
	
#ifdef __linux
	pthread_exit (NULL);
#else
	return;
#endif
}

/*! Function that takes care of receiving data from the Mote. It decodes the
    data which is received in a stream format through a serial port. The
		data is then sent to the appropriate functions for processing. It also
		starts the serial connection that can be used for sending packets to the
		mote. A second threat is started which takes care of transmitting
		packets to the mote.
*/
void StartMoteConnection()
{
#ifdef __linux
	pthread_t t1, t2;
  pthread_create (&t1, NULL, _StartMoteConnection, NULL);
  pthread_create (&t2, NULL, _StartMoteTransmit, NULL);
#else
	_beginthread(_StartMoteConnection,0,NULL);
	_beginthread(_StartMoteTransmit,0,NULL);
#endif
}


/*! This function sends a data packet adds a data packet to the queue used
		for transmitting messages through the serial communication port opened
    by the "ReceiveFromMote" function. Note that the 
*/
void SendToMote(char* data,int dataSz)
{
	memcpy(MCOMM_MSGBUF[MCOMM_MSGBUFPOS],data,dataSz);
	MCOMM_MSGBUFSZ[MCOMM_MSGBUFPOS] = dataSz;
	MCOMM_MSGBUFPOS = (MCOMM_MSGBUFPOS+1)%QUEUE_SIZE;

  return;
}


unsigned short crc_byte(unsigned short crc, unsigned char b)
{
	int i;

	crc = crc ^ (b << 8);
	for (i=0; i<8; i++)
	{
		if (crc & 0x8000)
			crc = (crc << 1) ^ 0x1021;
		else
			crc = crc << 1;
  }

	return crc;
}


unsigned short crc_packet(unsigned char *data, int len)
{
	int i;
	unsigned short crc = 0;

	for (i=0; i<len; i++) crc = crc_byte(crc, data[i]);

	return crc;
}


/*! This function adds the delimiters and the CRC bytes to the provided
    packet. It also removes special characters.
*/
unsigned int _CreateFinalMsg(unsigned char* out, unsigned char* in, unsigned int length)
{
	int i;
	unsigned int out_index;
	unsigned short crc;
	
	crc = crc_packet(in, length);
	
	out[0] = 0x7E;
	out_index = 1;

	for (i=0; i<length; i++)
	{
		if (in[i] == 0x7E)
		{
			out[out_index++] = 0x7D;
			out[out_index++] = 0x5E;
		}
		else if (in[i] == 0x7D)
		{
			out[out_index++] = 0x7D;
			out[out_index++] = 0x5D;
		}
		else
		{
			out[out_index++] = in[i];
		}
	}

	if( (crc & 0xFF) == 0x7E)
	{
		out[out_index++] = 0x7D;
		out[out_index++] = 0x5E;
	}
	else if( (crc & 0xFF) == 0x7D)
	{
		out[out_index++] = 0x7D;
		out[out_index++] = 0x5D;
	}
	else
		out[out_index++] = crc & 0xFF;
	
	if( (crc & 0xFF00) == 0x7E00)
	{
		out[out_index++] = 0x7D;
		out[out_index++] = 0x5E;
	}
	else if( (crc & 0xFF00) == 0x7D00)
	{
		out[out_index++] = 0x7D;
		out[out_index++] = 0x5D;
	}
	else
		out[out_index++] = (crc >> 8) & 0xFF;

	out[out_index++] = 0x7E;

	return out_index;
}


/*! It creates a TinyOS message to be transmitted over the serial port to
    the mote. It makes sure to encode the packet appropriately adding the
		CRC bytes and delimiters. It returns the size of the message.

		\param msg Pointer to the output message
		\param dest Destination to be transmitted. Use 65535 (i.e 0xFF) for
		  broadcast.
		\param pktType Type of cameraboard packet to be transmitted.
		\param data Pointer to the payload for the cameraboard packet.
		\param dataSz Size of the data to be transmitted.
*/
int CreateTinyOSMsg(char* msg, int dest, int pktType, char* data, int dataSz)
{
	char buf[1000];
	int msgSz = 0;

	// Generating the message before special characters are removed
	memset(buf,0,1000);
	buf[0] = 0x45;
	AMPKT_SET_DESTID(buf+SERIALPKT_HEADER_LENGTH,dest);
	AMPKT_SET_HANDLERID(buf+SERIALPKT_HEADER_LENGTH,HANDLERID_CITRIC);
	AMPKT_SET_PAYLOADLENGTH(buf+SERIALPKT_HEADER_LENGTH,
		dataSz+CBPKT_HEADER_LENGTH);
	CBPKT_SET_TYPE(buf+SERIALPKT_HEADER_LENGTH+AMPKT_HEADER_LENGTH,pktType);
	CBPKT_SET_ACTUALSRCID(buf+SERIALPKT_HEADER_LENGTH+AMPKT_HEADER_LENGTH,0);
	memcpy(buf+SERIALPKT_HEADER_LENGTH+AMPKT_HEADER_LENGTH+CBPKT_HEADER_LENGTH,
		data,dataSz*sizeof(char));

	// Getting the size before special characters are removed
	msgSz = dataSz+SERIALPKT_HEADER_LENGTH+AMPKT_HEADER_LENGTH+
		CBPKT_HEADER_LENGTH;

	// Generating final message without special character and with CRC
	msgSz = _CreateFinalMsg((unsigned char*)msg, (unsigned char*)buf, msgSz);

	return msgSz;
}
