#include "_ServerMenu.h"

int SMENU_MOTELIST[1000];
int SMENU_MOTELISTSZ = 0;
int SMENU_LOG_ON = 0;


/*! This function displays the menu options for the main menu and returns
    the input from the user.
*/
int InputMainMenu()
{
	int ch;
	printf("\n\n\n");
	printf("  MAIN MENU -------------------------\n");
	printf("  [%2d] Mote Selection                |\n",MENU_MAIN_SELECTION);
	printf("  [%2d] Display Packets               |\n",MENU_MAIN_DISPLAY);
	printf("  [%2d] Log Packets                   |\n",MENU_MAIN_LOG);
	printf("  [%2d] Send Packet                   |\n",MENU_MAIN_SEND);
	printf("  [%2d] Display Status                |\n",MENU_MAIN_STATUS);
	printf("  [%2d] RUN DEMO - BOUNDING BOXES     |\n",MENU_MAIN_RUNSAMPLE);
	printf("  [%2d] EXIT                          |\n",MENU_MAIN_EXIT);
	printf("  -----------------------------------\n");
	printf("  MAIN >> ");
	cin>>ch;
	printf("\n");

	return ch;
}


/*! This function displays the menu options for the display menu and returns
    the input from the user.
*/
int InputDisplayMenu()
{
	int ch;
	printf("\n");
	printf("    DISPLAY MENU ---------------------\n");
	printf("    [%2d] Messages as Dec              |\n",MENU_DISPLAY_DEC);
	printf("    [%2d] Messages as Hex              |\n",MENU_DISPLAY_HEX);
  printf("    [%2d] Messages ad Char             |\n",MENU_DISPLAY_CHAR);
	printf("    [%2d] Wireless Printf Only         |\n",MENU_DISPLAY_PRINTF);
	printf("    ----------------------------------\n");
	printf("    >> ");
	cin>>ch;
	printf("\n");

	return ch;
}


/*! This function interprets a string specifying a range of numbers with the
		format "a b:c d" (which includes the values 'a', 'd' and the range 'b' to
		'c') into a list of numbers that is stored in the SMENU_MOTELIST variable.

		\param buf Pointer to the buffer containing the string to be interpreted.
*/
void InterpretTextList(char* buf)
{
	char c = *(buf++);
	int n0 = 0;
	int nf = 0;

  int* listMotes = SMENU_MOTELIST;
	int listSz = 0;

	while(1)
	{
		if(c>47 && c<58)
			n0 = 10*n0 + ((int)c - 48);
		else if(c==' ' || c=='\0')
		{
			listMotes[listSz++] = n0;
			n0 = 0;

			if(c=='\0')
			{
				SMENU_MOTELISTSZ = listSz;
				return;
			}
		}
		else if(c==':')
		{
			nf = 0;
			c = *(buf++);
			while(c!='\0' && c!=' ')
			{
				if(c>47 && c<58)
					nf = 10*nf + ((int)c - 48);
				else
				{
					SMENU_MOTELISTSZ = 0;
					return;
				}
				c = *(buf++);
			}
			for(int i = n0; i<=nf; i++)
				listMotes[listSz++] = i;
			n0 = 0;
			if(c=='\0')
			{
				SMENU_MOTELISTSZ = listSz;
				return;
			}
		}
		else
		{
			SMENU_MOTELISTSZ = 0;
			return;
		}
		c = *(buf++);
	}

	return;
}


/*! This function displays the menu options for mote selection and returns
    the input from the user. It first display a list of currently selected
		motes and then gives options for updating this list. A range of motes can
		be specified within this function by using the syntax "a:b c", which
		includes the 'c' and the range between 'a' and 'b'. The selected motes
		are stored in the global list SMENU_MOTELIST of size SMENU_MOTELISTSZ.
*/
int InputMoteSelectionMenu()
{
	ifstream in;
	ofstream out;
	int ch;
	char tmp[1000];

  printf("\n");
  printf("[-] Current Mote Selection: ");
  for(int i = 0; i<SMENU_MOTELISTSZ; i++)
    printf(" %d",SMENU_MOTELIST[i]);
  printf("\n");

	printf("\n");
	printf("    MOTE SELECTION MENU --------------\n");
	printf("    [%2d] Load mote selection           |\n",MENU_SELECTION_LOAD);
	printf("    [%2d] Set mote selection            |\n",MENU_SELECTION_SET);
	printf("    [%2d] Save Selection                |\n",MENU_SELECTION_SAVE);
	printf("    ----------------------------------\n");
  printf("    >> ");
	cin>>ch;

	cin.getline(tmp,1000);
	switch(ch)
	{
	case MENU_SELECTION_LOAD:
		printf("    Name of file to load: ");
		cin.getline(tmp,1000);
		
		in.open(tmp);
		if(in.fail())
			printf("[-] FAILED\n");
		else
		{
			in.getline(tmp,1000);
			InterpretTextList(tmp);
		}
		in.close();

    printf("\n");
    printf("[-] Current Mote Selection: ");
    for(int i = 0; i<SMENU_MOTELISTSZ; i++)
      printf(" %d",SMENU_MOTELIST[i]);
    printf("\n");
		break;

	case MENU_SELECTION_SET:
		printf("    Enter selection (Use 65535 for broadcast): \n");
		printf("    ");
		cin.getline(tmp,1000);

		InterpretTextList(tmp);

    printf("\n");
    printf("[-] Current Mote Selection: ");
    for(int i = 0; i<SMENU_MOTELISTSZ; i++)
      printf(" %d",SMENU_MOTELIST[i]);
    printf("\n");
    break;

	case MENU_SELECTION_SAVE:
		printf("    Name of file to save: ");
		cin.getline(tmp,1000);

		out.open(tmp);
		if(out.fail())
			printf("[-] FAILED\n");
		else
		{
			for(int i = 0; i<SMENU_MOTELISTSZ; i++)
			{
				if(i>0)
					out<<" ";
				out<<SMENU_MOTELIST[i];
			}
			out<<endl;
			printf("[-] Selection Saved\n");
		}
		out.close();
		break;
	}
	printf("\n");

	return ch;
}


/*! This function displays the menu options for the log-file menu and returns
    the input from the user. This is meant to start or stop the log of all
		packets received into a file.
*/
int InputLogMenu()
{
	int ch;

  printf("\n");
  if(SMENU_LOG_ON)
    printf("[-] Log is RUNNING.\n");
  else
    printf("[-] Log is STOPPED.\n");
	printf("\n");

  printf("    LOG MENU -------------------------\n");
	printf("    [%2d] START log                     |\n",MENU_LOG_START);
	printf("    [%2d] STOP log                      |\n",MENU_LOG_STOP);
	printf("    ----------------------------------\n");
	printf("    >> ");
	cin>>ch;

  switch(ch)
  {
  case MENU_LOG_START: SMENU_LOG_ON = 1; break;
  case MENU_LOG_STOP: SMENU_LOG_ON = 0; break;
  }

	printf("\n");
  if(SMENU_LOG_ON)
    printf("[-] Log is RUNNING.\n");
  else
    printf("[-] Log is STOPPED.\n");
	printf("\n");

	return ch;
}


/*! This function displays the menu options for the SEND menu and returns
    the input from the user. It is used for sending different types of messages
		to the motes.
*/
int InputSendMenu()
{
	int ch;

	printf("\n");
  printf("[-] Current Mote Selection: ");
  for(int i = 0; i<SMENU_MOTELISTSZ; i++)
    printf(" %d",SMENU_MOTELIST[i]);
  printf("\n");

  printf("\n");
  printf("    SEND MENU ------------------------\n");
	printf("    [%2d] Stop Synch                    |\n",MENU_SEND_STOPSYNCH);
	printf("    [%2d] Ping                          |\n",MENU_SEND_PING);
	printf("    [%2d] Set Parameters (*)            |\n",MENU_SEND_SETPARAM);
	printf("    [%2d] Set Wireless Printf           |\n",MENU_SEND_SETPRINTF);
	printf("    [%2d] Send from file (*)            |\n",MENU_SEND_FILE);
	printf("    ----------------------------------\n");
	printf("    (*) NOT IMPLEMENTED YET\n");
	printf("    >> ");
	cin>>ch;

	return ch;
}