#include "_TaskManager.h"

#ifdef __linux
	#include <string.h>
	#include <pthread.h>
	#include <sys/timeb.h>
	
	#define DWORD long
#else
	#include <windows.h>
#endif

int TASKM_DEBUG_DISPLAY;
moteStates TASKM_STATES;

/*! Array storing the packets received. */
xshort TASKM_PACKETBUF[QUEUE_SIZE][MAX_PACKET_LEN];
/*! Variable storing the position to add the next packet to the buffer. */
int TASKM_PACKETBUFPOS = 0;

/*! Variables for file transfer */
int CHUNKSIZE = CBPKT_MAX_PAYLOAD - 2;
FILE* infile;
unsigned int infile_size = 0;
char infile_name[16];
int infile_is_busy = 0;

#ifdef __linux
	struct timeb timeA, timeB;
#else
	DWORD timeA, timeB;
#endif
int timeDiff;
float transferRate;

/*! Constructor of the class. It makes sure that all necessary variables are
		set to zero at initial state.
*/
moteStates::moteStates()
{
	memset(m_active,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_debugSend,0,sizeof(xshort)*MAX_MOTES_NUMBER);

	memset(m_timestamp,0,sizeof(long)*MAX_MOTES_NUMBER);
	memset(m_cpuFreq,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_inqueueSize,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_outqueueSize,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_upTime,0,sizeof(int)*MAX_MOTES_NUMBER);
	memset(m_resolution,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_color,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_autoExposure,0,sizeof(xshort)*MAX_MOTES_NUMBER);
	memset(m_exposureValue,0,sizeof(xshort)*MAX_MOTES_NUMBER);

	return;
}


/*! It displays the status of the active motes in text format.
*/
void moteStates::display()
{
	struct tm * t;

	printf("****************************************************\n");
	printf("ID  | TIME     | CPU | IN-Q | OUT-Q | RESOL\n");
	printf("****************************************************\n");
	for(int i = 0; i<MAX_MOTES_NUMBER; i++)
		if(m_active[i])
		{
			t = localtime(&TASKM_STATES.m_lastTimeReceived[i]);
			printf("%03d | %02d:%02d:%02d | %03d |  %03d |   %03d |   %03d\n",
				i,t->tm_hour,t->tm_min,t->tm_sec,TASKM_STATES.m_cpuFreq[i],
				TASKM_STATES.m_inqueueSize[i],TASKM_STATES.m_outqueueSize[i],
				TASKM_STATES.m_resolution[i]);
		}

	return;
}


/*! This function takes the provided message and adds it to the queue for the
		task manager.

		\param msg Pointer to the message to be added to the queue.
		\param msgSz Size of the message.
*/
void ProcessPacket(xshort* msg, int msgSz)
{
	memcpy(TASKM_PACKETBUF[TASKM_PACKETBUFPOS],msg,msgSz);
	TASKM_PACKETBUFPOS = (TASKM_PACKETBUFPOS+1)%QUEUE_SIZE;
}

#ifdef __linux
	void* _StartTaskManager(void* parm)
#else
	void _StartTaskManager(void* parm)
#endif
{
	int pktToProcess = 0; // Keeps track of the packet to be processed.
	xshort* msg; // Pointer to the message
	xshort* pkt; // Pointer to the packet
	int pktSz; // Size of the packet
	int pktType; // Type of the packet
	int moteID; // Mote ID

	while(1)
	{
		// Waits while the next packet to be processed has not been added yet.
		while(pktToProcess==TASKM_PACKETBUFPOS)
		{
#ifdef __linux
			sleep(1);
#else
			Sleep(1000);
#endif
		}

		// Getting message
		msg = TASKM_PACKETBUF[pktToProcess];

		if(MSG_IS_CB_PKT(msg))
		{
			// Getting packet information
			moteID = MSG_GET_MOTEID(msg);
			pktType = MSG_GET_PKTTYPE(msg);
			pktSz = MSG_GET_PKTSZ(msg);
			pkt = MSG_GET_PKT(msg);

			// Making Mote Active no matter what message we get.
			TASKM_STATES.m_active[moteID] = 1;
			TASKM_STATES.m_lastTimeReceived[moteID] = time(NULL);

			// Passing packet to the the appropriate process.
			switch(pktType)
			{
			case PACKET_PING_IN:
				ProcessPing(moteID,pkt,pktSz);
				break;
			case PACKET_PRINTF_IN:
				ProcessDebug(moteID,pkt,pktSz);
				break;
			case PACKET_FILETRANSFER_INFO:
				ProcessFileInfo(moteID,pkt,pktSz);
				break;
			case PACKET_FILETRANSFER_DATA:
				ProcessFile(moteID,pkt,pktSz);
				break;
			default:
				ProcessUser(pktType,moteID,pkt,pktSz);
			}
		}

		// Pointing to the next packet.
		pktToProcess = (pktToProcess+1)%QUEUE_SIZE;
	}
	
#ifdef __linux
	pthread_exit(NULL);
#else
	return;
#endif
}


/*! This function starts the task manager. The task manager runs in the
		background and takes care of processing packets received from the motes.
		It makes sure to pass the information to the appropriate running task.
*/
void StartTaskManager()
{
#ifdef __linux
	pthread_t t;
  pthread_create(&t, NULL, _StartTaskManager, NULL);
#else
	_beginthread(_StartTaskManager,0,NULL);
#endif
}


/*! This function takes care of processing the pings received from the motes.
		It updates corresponding information in the mote states.

		\param moteID The ID of the mote.
		\param pkt Pointer to the content of the packet containing ping info.
		\param pktSz Size of the packet.
*/
void ProcessPing(int moteID, xshort* pkt, int pktSz)
{
	if(moteID>=MAX_MOTES_NUMBER || moteID<0)
		return;

	// Recovering Timestamp
	long timestamp = 0;
	for(int i = 0; i<8; i++)
		timestamp = timestamp*256 + *(pkt++);
	TASKM_STATES.m_timestamp[moteID] = timestamp;

	TASKM_STATES.m_cpuFreq[moteID] = *(pkt++);
	TASKM_STATES.m_inqueueSize[moteID] = *(pkt++);
	TASKM_STATES.m_outqueueSize[moteID] = *(pkt++);

	// Recovering Up Time
	int upTime = 0;
	for(int i = 0; i<4; i++)
		upTime = upTime*256 + *(pkt++);
	TASKM_STATES.m_upTime[moteID] = upTime;

	TASKM_STATES.m_resolution[moteID] = *(pkt++);
	TASKM_STATES.m_color[moteID] = *(pkt++);
	TASKM_STATES.m_autoExposure[moteID] = *(pkt++);
	TASKM_STATES.m_exposureValue[moteID] = *(pkt++);

	return;
}


/*! This function takes care of processing the debug messages received from
		the mote.

		\param moteID The ID of the mote.
		\param pkt Pointer to the content of the packet containing ping info.
		\param pktSz Size of the packet.
*/
void ProcessDebug(int moteID, xshort* pkt, int pktSz)
{
	// For safety we make the last entry be a zero to end the string.
	pkt[pktSz-1] = 0;

	// We display the text string
	if(TASKM_DEBUG_DISPLAY)
		printf("[%03d]: %s\n",moteID,pkt);

	return;
}

/*! This function takes care of processing the file info messages received from
		the mote.

		\param moteID The ID of the mote.
		\param pkt Pointer to the content of the packet containing ping info.
		\param pktSz Size of the packet.
*/
void ProcessFileInfo(int moteID, xshort* pkt, int pktSz)
{
	unsigned char type = pkt[0];

	switch (type)
	{
	case 0: // start of file
		if (infile_is_busy)
		{
			fclose(infile);
		}
		infile_size = ((unsigned int)pkt[1] << 24) + ((unsigned int)pkt[2] << 16) + ((unsigned int)pkt[3] << 8) + (unsigned int)pkt[4];
		memcpy(infile_name, pkt+5, pktSz-5);
		infile_name[pktSz-5] = 0;

		infile = fopen(infile_name, "wb");
		if (infile == NULL)
		{
			return;
		}
		infile_is_busy = 1;

#ifdef __linux
		ftime(&timeA);
#else
		timeA = GetTickCount();
#endif
		printf("receiving file %s from mote %d, size = %d\n", infile_name, moteID, infile_size);
		
		break;
	case 1: // end of file
		if (infile_is_busy)
		{
#ifdef __linux
			ftime(&timeB);
			timeDiff = (timeB.time - timeA.time)*1000 + (timeB.millitm - timeA.millitm);
#else
			timeB = GetTickCount();
			timeDiff = timeB - timeA;
#endif
			transferRate = infile_size / ((float)timeDiff / 1000);
			printf("\nwrote file %s from mote %d, transfer rate = %.0f B/s\n", infile_name, moteID, transferRate);
			fclose(infile);
		}

		infile_is_busy = 0;

		break;
	default:
		break;
	}

	return;
}

/*! This function takes care of processing file messages received from
		the mote.

		\param moteID The ID of the mote.
		\param pkt Pointer to the content of the packet containing ping info.
		\param pktSz Size of the packet.
*/
void ProcessFile(int moteID, xshort* pkt, int pktSz)
{
	int segmentnum = ((unsigned int)pkt[0] << 8) + (unsigned int)pkt[1];

	if (infile_is_busy)
	{
		fseek(infile, segmentnum*CHUNKSIZE, SEEK_SET);
		fwrite(pkt+2, 1, pktSz-2, infile);
		printf(".");
	}

	return;
}
