#ifndef TASKMANAGER_H
#define TASKMANAGER_H

#include <time.h>

#ifdef __linux
	#include <pthread.h>
#else
#endif

#include "_MoteComm.h"
#include "../C-Common/_PacketTypes.h"

/*! Class used for storing the states of a motes. */
class moteStates
{
public:
	/*! Flag specifying if a mote is active or not. */
	xshort m_active[MAX_MOTES_NUMBER];
	/*! Flags specifying which mote will receive debug information. */
	xshort m_debugSend[MAX_MOTES_NUMBER];

	/*! Time in which last message was received. */
	time_t m_lastTimeReceived[MAX_MOTES_NUMBER];

	long m_timestamp[MAX_MOTES_NUMBER]; //!< Global Time received from mote
	xshort m_cpuFreq[MAX_MOTES_NUMBER]; //!< CPU frequencey received from mote
	xshort m_inqueueSize[MAX_MOTES_NUMBER]; //!< Size of Incoming queue in mote
	xshort m_outqueueSize[MAX_MOTES_NUMBER]; //!< Size of outgoing queue in mote
	int m_upTime[MAX_MOTES_NUMBER]; //!< Cameraboard uptime in minutes in mote
	xshort m_resolution[MAX_MOTES_NUMBER]; //!< Camera resolution mode in mote
	xshort m_color[MAX_MOTES_NUMBER]; //!< Camera color mode received from mote
	xshort m_autoExposure[MAX_MOTES_NUMBER]; //!< State of auto exposure in mote
	xshort m_exposureValue[MAX_MOTES_NUMBER]; //!< Camera exposure value in mote
	
	moteStates();
	void display();
};

/*! Global variable storing display mode for the streamed packets. */
extern int TASKM_DEBUG_DISPLAY;
/*! Global variable storing the state information of each mote. */
extern moteStates TASKM_STATES;

void StartTaskManager();
void ProcessPing(int moteID, xshort* pkt, int pktSz);
void ProcessDebug(int moteID, xshort* pkt, int pktSz);
void ProcessFileInfo(int moteID, xshort* pkt, int pktSz);
void ProcessFile(int moteID, xshort* pkt, int pktSz);
void ProcessUser(int pktType, int moteID, xshort* pkt, int pktSz);

#endif
